/*
 * Copyright (c) 2003 David E. Bakken, his research students, and Washington State University.
 * Please see the file LICENSE.pdf for more details on terms of use.
 */

package ZEN_IDL.be;

import java.io.*;
import java.util.*;
import util.*;

/*********************************************************
* Generates Config_macros.m4 file
* This is the configuration file that is used by the M4 macro parser
* to enable/disable java code as needed.
*
* @author David McKinnon
*********************************************************/
public class MQCConfigMacrosGenerator
{
  private MQCCodeGenHelper codeGenHelper; /* The code generator helper object */
  private MQCConfig config; /* The configuration options */
    
  /*********************************************************
  * Constructor. Initializes variables.
  * 
  *********************************************************/
  public MQCConfigMacrosGenerator(MQCConfig config)
  {
    codeGenHelper = new MQCCodeGenHelper(config);    
    this.config = config;
  }
  
  /*********************************************************
  * Generates the Config.java file
  *
  *********************************************************/
  public void generate()
  {
    PrintWriter pw = codeGenHelper.makeFileExt("", "Config_macros", ".m4");

    pw.println("divert(-1)");

    pw.print("define(`__J2SE__'");
    if (config.getCLDCStatus())
      pw.println(")dnl");
    else
      pw.println(", `$*')dnl");

    pw.print("define(`__CLDC__'");
    if (config.getCLDCStatus())
      pw.println(", `$*')dnl");
    else
      pw.println(")dnl");

    //
    // Configure the Transports
    //--------------------------
    Vector transports = config.getServerTransport();

    pw.print("define(`__TCP_TP__'");
    if(transports.contains(Constants.TR_TCP_STR))
      pw.println(", `$*')dnl");
    else
      pw.println(")dnl");

    pw.print("define(`__UDP_TP__'");
    if((transports.contains(Constants.TR_UNRELIABLE_UDP_STR)) ||
      (transports.contains(Constants.TR_GOBACKN_UDP_STR)) ||
      (transports.contains(Constants.TR_STOPANDWAIT_UDP_STR)))
      pw.println(", `$*')dnl");
    else
      pw.println(")dnl");
    
    pw.print("define(`__UNRELIABLE_UDP__'");
    if(transports.contains(Constants.TR_UNRELIABLE_UDP_STR))
    {
      pw.println(", `$*')dnl");
      pw.println("define(`__MAX_UDP_DATAGRAM_SIZE__', 1024)dnl");
    }
    else
       pw.println(")dnl");

    pw.print("define(`__GO_BACK_N_UDP__'");
    if(transports.contains(Constants.TR_GOBACKN_UDP_STR))
    {
      pw.println(", `$*')dnl");
      pw.println("define(`__MAX_UDP_DATAGRAM_SIZE__', 1024)dnl");
    }
    else
       pw.println(")dnl");
       
    pw.print("define(`__STOP_N_WAIT_UDP__'");
    if(transports.contains(Constants.TR_STOPANDWAIT_UDP_STR))
    {
      pw.println(", `$*')dnl");
      pw.println("define(`__MAX_UDP_DATAGRAM_SIZE__', 1024)dnl");
    }
    else
       pw.println(")dnl");

    pw.print("define(`__SERIAL_TP__'");
    if(transports.contains(Constants.TR_SERIAL_STR))
      pw.println(", `$*')dnl");
    else
      pw.println(")dnl");

    pw.print("define(`__ONEWIRE_TP__'");
    if(transports.contains(Constants.TR_1WIRE_STR))
      pw.println(", `$*')dnl");
    else
      pw.println(")dnl");
    
    //
    // Configure the Protocols
    //--------------------------
    Vector protocols = config.getServerProtocol();
    
    pw.print("define(`__GIOP_PROTOCOL__'");
    if(protocols.contains(Constants.GIOP_GIOP_STR))
      pw.println(", `$*')dnl");
    else
      pw.println(")dnl");

    pw.print("define(`__GIOPLITE_PROTOCOL__'");
    if(protocols.contains(Constants.GIOP_LITE_STR))
      pw.println(", `$*')dnl");
    else
      pw.println(")dnl");

    pw.print("define(`__MQCIOP_PROTOCOL__'");
    if(protocols.contains(Constants.GIOP_MQC_STR))
      pw.println(", `$*')dnl");
    else
      pw.println(")dnl");
    
    pw.print("define(`__GIOP10__'");
    if(config.supportsGIOPVersion(Constants.GIOP_V10_STR))
      pw.println(", `$*')dnl");
    else
      pw.println(")dnl");
      
    pw.print("define(`__GIOP11__'");
    if(config.supportsGIOPVersion(Constants.GIOP_V11_STR))
      pw.println(", `$*')dnl");
    else
      pw.println(")dnl");
      
    pw.print("define(`__GIOP12__'");
    if(config.supportsGIOPVersion(Constants.GIOP_V12_STR))
      pw.println(", `$*')dnl");
    else
      pw.println(")dnl");
      
    pw.print("define(`__GIOP13__'");
    if(config.supportsGIOPVersion(Constants.GIOP_V13_STR))
      pw.println(", `$*')dnl");
    else
      pw.println(")dnl");
      
    pw.print("define(`__FIX_SIZE__'");
    if(config.getMSGPayload() != 0)
      pw.println(", `$*')dnl");
    else
      pw.println(")dnl");
      
    pw.print("define(`__VAR_SIZE__'");
    if(config.getMSGPayload() == 0)
      pw.println(", `$*')dnl");
    else
      pw.println(")dnl");
      
    
    // here check to see if we have any defines for Temporal Redundancy
    String strTemporal = config.getFTTemporal();
    if(strTemporal.equalsIgnoreCase("1")) {
        // here no temporal redundancy is needed
        pw.println("define(`__TEMPORAL__')dnl");
        pw.println("define(`__NOTEMPORAL__', `$*')dnl");
    } else {
        // here temporal redundancy is needed
        pw.println("define(`__NOTEMPORAL__')dnl");
        pw.println("define(`__TEMPORAL__', `$*')dnl");
    }
    
    // check to see if we have any defines for Spatial Redundancy
    Hashtable htSpatial = config.getFTSpatial();
    String strSpatialEnabled = (String) htSpatial.get("enableSpatial");
    if(strSpatialEnabled.equalsIgnoreCase("true")) {
        // spatial redundancy has been enabled
        pw.println("define(`__NOSPATIAL__')dnl");
        pw.println("define(`__SPATIAL__', `$*')dnl");
    } else {
        // spatial redundancy has not been enabled
        pw.println("define(`__SPATIAL__')dnl");
        pw.println("define(`__NOSPATIAL__', `$*')dnl");
    }
    
    // make the define for the common code between spatial and temporal
    if(strSpatialEnabled.equalsIgnoreCase("true") || !(strTemporal.equalsIgnoreCase("1"))) {
        pw.println("define(`__TEMPSPACE__', `$*')dnl");
        pw.println("define(`__NOTEMPSPACE__')dnl");
    } else {
        pw.println("define(`__TEMPSPACE__')dnl");
        pw.println("define(`__NOTEMPSPACE__', `$*')dnl");
    }
    
    // here check to see if we have any defines for GroupCom
    Hashtable htGroupCom = config.getFTGroupCom();
    // get the string that represents whether Group Com is enabled
    String strGCEnabled = (String) htGroupCom.get("enableGroupCom");
    // get the string that represents whether uniform or non uniform Group Com 
    //  is enabled
    String strGCUniform = (String) htGroupCom.get("ftGCUniform");
    if(strGCEnabled.equalsIgnoreCase("true")) {
        pw.println("define(`__GROUPCOM__', `$*')dnl");
        pw.println("define(`__NOGROUPCOM__')dnl");
        if(strGCUniform.equalsIgnoreCase("uniform")) {
            // in this case uniform group com is selected
            pw.println("define(`__GCUNIFORM__', `$*')dnl");
            pw.println("define(`__GCNONUNIFORM__')dnl");
        } else {
            // in this case nonuniform group com is selected
            pw.println("define(`__GCNONUNIFORM__', `$*')dnl");
            pw.println("define(`__GCUNIFORM__')dnl");
        }
    } else {
        // no group com has been selected
        pw.println("define(`__GROUPCOM__')dnl");
        pw.println("define(`__NOGROUPCOM__', `$*')dnl");
        pw.println("define(`__GCUNIFORM__')dnl");
        pw.println("define(`__GCNONUNIFORM__')dnl");
    }
    
    // check value if we have any value redundancy defines
    String strEnableValue = config.getFTValueRed();
    if(strEnableValue.equalsIgnoreCase("true")) {
        // value redundancy has been enabled
        pw.println("define(`__NOVALUERED__')dnl");
        pw.println("define(`__VALUERED__', `$*')dnl");
    } else {
        // value redundancy has not been enabled
        pw.println("define(`__VALUERED__')dnl");
        pw.println("define(`__NOVALUERED__', `$*')dnl");
    }
    
    // set the security defines as needed
    if(config.getSecurityCipherStatus()) {
        // security has been enabled
        pw.println("define(`__SECURITY__', `$*')dnl");
        pw.println("define(`__SEC_CIPHER_ALG__', `" + 
                               config.getSecurityCipher() + "')dnl");
        //-- Cipher Mode --
        String cMode = config.getSecurityCipherMode();
        pw.println("define(`__SEC_CIPHER_MODE__', `" + cMode + "')dnl");
        if (cMode.equalsIgnoreCase("ECB")) {
            pw.println("define(`__SEC_CIPHER_MODE_ECB__', `$*')dnl");
        } else {
            pw.println("define(`__SEC_CIPHER_MODE_ECB__')dnl");
        }
        if (cMode.equalsIgnoreCase("CBC")) {
            pw.println("define(`__SEC_CIPHER_MODE_CBC__', `$*')dnl");
        } else {
            pw.println("define(`__SEC_CIPHER_MODE_CBC__')dnl");
        }
        //-- Cipher Padding --
        String cPadding = config.getSecurityCipherPadding();
        pw.println("define(`__SEC_CIPHER_PADDING__', `" + cPadding + "')dnl");
        if (cPadding.equalsIgnoreCase("NONE")) {
            pw.println("define(`__SEC_CIPHER_PADDING_NONE__', `$*')dnl");
        } else {
            pw.println("define(`__SEC_CIPHER_PADDING_NONE__')dnl");
        }
        if (cPadding.equalsIgnoreCase("PKCS5")) {
            pw.println("define(`__SEC_CIPHER_PADDING_PKCS5__', `$*')dnl");
        } else {
            pw.println("define(`__SEC_CIPHER_PADDING_PKCS5__')dnl");
        }
        pw.println("define(`__SEC_CIPHER_KEY_LEN__', `" + 
                               config.getSecurityCipherKeyLen()/8 + "')dnl");
    } else {
        // security has not been enabled
        pw.println("define(`__SECURITY__')dnl");
        pw.println("define(`__SEC_CIPHER_ALG__')dnl");
        pw.println("define(`__SEC_CIPHER_MODE__')dnl");
        pw.println("define(`__SEC_CIPHER_MODE_ECB__')dnl");
        pw.println("define(`__SEC_CIPHER_MODE_CBC__')dnl");
        pw.println("define(`__SEC_CIPHER_PADDING__')dnl");
        pw.println("define(`__SEC_CIPHER_PADDING_NONE__')dnl");
        pw.println("define(`__SEC_CIPHER_PADDING_PKCS5__')dnl");
        pw.println("define(`__SEC_CIPHER_KEY_LEN__')dnl");
    }
    
    if (config.getSecurityMDStatus()) {
        pw.println("define(`__SECURITY_MD__', `$*')dnl");
        pw.println("define(`__SEC_MD_ALG__', `" + config.getSecurityMD() + "')dnl");
    } else {
        pw.println("define(`__SECURITY_MD__')dnl");
        pw.println("define(`__SEC_MD_ALG__')dnl");
    }

    if (config.getSecurityMDStatusNotHMAC()) {
        pw.println("define(`__SECURITY_MD_NOT_HMAC__', `$*')dnl");
    } else {
        pw.println("define(`__SECURITY_MD_NOT_HMAC__')dnl");
    }
    
    if (config.getSecurityMACStatus()) {
        pw.println("define(`__SECURITY_MAC__', `$*')dnl");
        pw.println("define(`__SEC_MAC_ALG__', `" + config.getSecurityMAC() + "')dnl");
        if (config.getSecurityMAC().equalsIgnoreCase("HMAC")) {
            pw.println("define(`__SEC_MAC_ALG_PARAMS__', `msgDigest')dnl");
        }else {
            pw.println("define(`__SEC_MAC_ALG_PARAMS__')dnl");
        }
        pw.println("define(`__SEC_MAC_KEY_LEN__', `" + 
                               config.getSecurityMACKeyLen()/8 + "')dnl");
    } else {
        pw.println("define(`__SECURITY_MAC__')dnl");
        pw.println("define(`__SEC_MAC_ALG__')dnl");
        pw.println("define(`__SEC_MAC_ALG_PARAMS__')dnl");
        pw.println("define(`__SEC_MAC_KEY_LEN__')dnl");
    }
    
    if (config.getSecurityORBKeyStatus()) {
        pw.println("define(`__SECURITY_ORBKEY__', `$*')dnl");
    } else {
        pw.println("define(`__SECURITY_ORBKEY__')dnl");
    }

    if (config.getSecurityAuditStatus()) {
        pw.println("define(`__SECURITY_AUDIT__', `$*')dnl");
        pw.println("define(`__SEC_AUDIT_ALG__', `" + config.getSecurityAudit() + "')dnl");
        pw.println("define(`__SEC_AUDIT_LOG_LEN__', `" + 
                               config.getSecurityAuditLogLen() + "')dnl");
    } else {
        pw.println("define(`__SECURITY_AUDIT__')dnl");
        pw.println("define(`__SEC_AUDIT_ALG__')dnl");
        pw.println("define(`__SEC_AUDIT_LOG_LEN__')dnl");
    }
        
    // Debug macro
    pw.print("define(`__DEBUG__'");
    if (config.getDebugStatus())
      pw.println(", `$*')dnl");
    else
      pw.println(")dnl");

    // Debug level macros (upto 5 (hard coded) levels are supported)
    for (int i = 1;  i < 6;  i++) {
      pw.print("define(`__DEBUG" + i + "__'");
      if (config.getDebugStatus() && i <= config.getDebugLevel())
        pw.println(", `$*')dnl");
      else
        pw.println(")dnl");
    }
    
    // Debug memory macro
    pw.print("define(`__DEBUG_MEMORY__'");
    if (config.getDebugMemoryStatus())
      pw.println(", `$*')dnl");
    else
      pw.println(")dnl");
      
    // Debug timing macro
    pw.print("define(`__DEBUG_TIMING__'");
    if (config.getDebugTimingStatus())
      pw.println(", `$*')dnl");
    else
      pw.println(")dnl");

    // Platform specific macros, etc
    pw.println("define(`__OS__', `LINUX')dnl");
    pw.println("ifelse(__OS__, `LINUX', `");
    pw.println("  define(`__LINUX__', `$*')dnl");
    pw.println("  define(`__TIMER_INIT__', `JNITimer.init(1000000)')dnl");
    pw.println("  define(`__GET_TIME__', `JNITimer.currentTime()')dnl");
    pw.println("  define(`__TICKS_PER_MS__', `1000');dnl");
    pw.println("  define(`__SS_ITERATIONS__', `15000')dnl");
    pw.println("  define(`__SS_LOOP_CNT__', `3')dnl");
    pw.println("  define(`__MAIN_LOOP_CNT__', `3')dnl");
    pw.println("  define(`__TIMEOUT_SEC__', `600')dnl");
    pw.println("', `");
    pw.println("  define(`__LINUX__')dnl");
    pw.println("')dnl");
    pw.println("ifelse(__OS__, `DARWIN', `");
    pw.println("  define(`__DARWIN__', `$*')dnl");
    pw.println("  define(`__TIMER_INIT__', `;')dnl");
    pw.println("  define(`__GET_TIME__', `JNITimer.currentTime()')dnl");
    pw.println("  define(`__TICKS_PER_MS__', `1000');dnl");
    pw.println("  define(`__SS_ITERATIONS__', `15000')dnl");
    pw.println("  define(`__SS_LOOP_CNT__', `3')dnl");
    pw.println("  define(`__MAIN_LOOP_CNT__', `3')dnl");
    pw.println("  define(`__TIMEOUT_SEC__', `600')dnl");
    pw.println("', `");
    pw.println("  define(`__DARWIN__')dnl");
    pw.println("')dnl");
    pw.println("ifelse(__OS__, `GENERIC', `");
    pw.println("  define(`__GENERIC__', `$*')dnl");
    pw.println("  define(`__TIMER_INIT__', `;')dnl");
    pw.println("  define(`__GET_TIME__', `System.currentTimeMillis()')dnl");
    pw.println("  define(`__TICKS_PER_MS__', `1');dnl");
    pw.println("  define(`__SS_ITERATIONS__', `15000')dnl");
    pw.println("  define(`__SS_LOOP_CNT__', `3')dnl");
    pw.println("  define(`__MAIN_LOOP_CNT__', `3')dnl");
    pw.println("  define(`__TIMEOUT_SEC__', `600')dnl");
    pw.println("', `");
    pw.println("  define(`__GENERIC__')dnl");
    pw.println("')dnl");
    pw.println("ifelse(__OS__, `SAJE', `");
    pw.println("  define(`__SAJE__', `$*')dnl");
    pw.println("  define(`__TIMER_INIT__', `;')dnl");
    pw.println("  define(`__GET_TIME__', `com.ajile.jem.rawJEM.getTime()')dnl");
    pw.println("  define(`__TICKS_PER_MS__', `1000');dnl");
    pw.println("  define(`__SS_ITERATIONS__', `100')dnl");
    pw.println("  define(`__SS_LOOP_CNT__', `3')dnl");
    pw.println("  define(`__MAIN_LOOP_CNT__', `3')dnl");
    pw.println("  define(`__TIMEOUT_SEC__', `600')dnl");
    pw.println("', `");
    pw.println("  define(`__SAJE__')dnl");
    pw.println("')dnl");
    pw.println("ifelse(__OS__, `TINI', `");
    pw.println("  define(`__TINI__', `$*')dnl");
    pw.println("  define(`__TIMER_INIT__', `;')dnl");
    pw.println("  define(`__GET_TIME__', `com.dalsemi.system.TINIOS.uptimeMillis()')dnl");
    pw.println("  define(`__TICKS_PER_MS__', `1');dnl");
    pw.println("  define(`__SS_ITERATIONS__', `10')dnl");
    pw.println("  define(`__SS_LOOP_CNT__', `3')dnl");
    pw.println("  define(`__MAIN_LOOP_CNT__', `3')dnl");
    pw.println("  define(`__TIMEOUT_SEC__', `900')dnl");
    pw.println("', `");
    pw.println("  define(`__TINI__')dnl");
    pw.println("')dnl");

    pw.println("undefine(`index')dnl");
    pw.println("undefine(`len')dnl");
    pw.println("divert");
    pw.println("/* Do NOT edit this file--It was autogenerated by m4 from a *.java.m4 file */");

    pw.flush();
    pw.close();
  }
}
