/*
 * Copyright (c) 2003 David E. Bakken, his research students, and Washington State University.
 * Please see the file LICENSE.pdf for more details on terms of use.
 */

package ZEN_IDL.be;

import java.util.*;
import ZEN_IDL.ast.*;
import ZEN_IDL.be.helperUtils.*;
import ZEN_IDL.be.symbolTableUtils.*;
import util.*;

/*********************************************************
* Checks the symbol table against the configuration
* options.
*
* @author Olav Haugan
*
* Date: 14.10.01
*********************************************************/
public class MQCConstraintCheckerVisitor extends AbstractVisitor
{
	private MQCConfig config;
	private int noInterfaces;
	private int noMethods;
		
	/*********************************************************
	* Constructor. Initializes variables.
	*
	* @param MQCConfig The configuration options
	*********************************************************/
	public MQCConstraintCheckerVisitor(MQCConfig config)
	{
		this.config = config;
		this.noInterfaces = 0;
		this.noMethods = 0;
	}

	/*********************************************************
	* Visitor function called for each module key word.
	* Sets the current node type.
	*
	* @param ASTmodule The module node
	*
	* @param Object The data object
	*********************************************************/
	public Object visit(ASTmodule node, Object data)
	{
		VisitorState mState = (VisitorState) data;
		mState.setCurrentNodeType( IDLParserTreeConstants.JJTMODULE );
		data = node.childrenAccept(this, data);
		mState.stkPackages.pop();
		return data;
	}

	/*********************************************************
	* Visitor function called for each module definition.
	*
	* @param ASTidentifier The module node
	*
	* @param Object The data object
	*********************************************************/
	public Object visit(ASTidentifier node, Object data)
	{
		VisitorState mState = (VisitorState) data;
		
		String name = node.getFirstToken().toString(); 	
		if(mState.getCurrentNodeType() == IDLParserTreeConstants.JJTMODULE)
		{
			//name of module/package/directory
			mState.stkPackages.push( name );
			//Create new package directory
			mState.setCurrentNodeType( -1 );
		}
		data = node.childrenAccept(this, data);
		return data;
	}
	
	/*********************************************************
	* Visitor function called for each interface definition.
	*
	* @param ASTinterface_dcl The module node
	*
	* @param Object The data object
	*********************************************************/
	public Object visit(ASTinterface_dcl node, Object data)
	{
		VisitorState mState = (VisitorState) data;

		String packageName = mState.getPackage();
		String interfaceName = node.jjtGetChild(0).jjtGetChild(0).getFirstToken().toString();
		Vector interfaceMethods = SymbolTable.getSymbol( interfaceName, packageName ).getMethods();
		
		noInterfaces++;
		noMethods += interfaceMethods.size();
		
		if(!checkDataTypes(interfaceMethods))
			System.exit(-1);
		if(!checkPayload(interfaceMethods))
			System.exit(-1);
		if(!checkNoInterfaces())
		{
			System.out.println("Number of interfaces is too high");
			System.exit(-1);
		}
		if(!checkNoMethods())
		{
			System.out.println("Number of methods is too high");
			System.exit(-1);
		}
		return data;
	}
	
	/*********************************************************
	* Visitor function called for each exception definition.
	*
	* @param ASTexcept_dcl The module node
	*
	* @param Object The data object
	*********************************************************/
  public Object visit(ASTexcept_dcl node, Object data)
  {
		if(!config.isValidDataType(Constants.EXCEPTION_TYPE[1]))
		{
			System.out.println("User exceptions not allowed in current profile");
			System.exit(-1);
		}
		data = node.childrenAccept(this, data);
		return data;
  }
	
	/*********************************************************
	* Visitor function called for each exception variable
	*
	* @param ASTbase_type_spec The variable def.
	*
	* @param Object The data object
	*********************************************************/
	public Object visit( ASTbase_type_spec node, Object data )
	{
		VisitorState currentState = (VisitorState)data;
		Token firstToken = node.getFirstToken();
		Token lastToken = node.getLastToken();
		String type_spec = firstToken.toString();
		while( firstToken != lastToken )
		{
			firstToken = firstToken.next;
			type_spec += " " + firstToken.toString();
		}
		if(!config.isValidDataType(type_spec))
		{
			System.out.println(type_spec + " not allowed.");
			System.exit(-1);
		}
		return data;
	}
	
	/*********************************************************
	* Visitor function for a IDL type defs. Called
	* by the visitor pattern for each typedef declared.
	*
	* @param ASTtype_declarator The type def node from the parse tree
	*        
	* @param Object The data object
	*********************************************************/
	public Object visit(ASTtype_declarator node, Object data )
	{
		data = node.childrenAccept( new XTypedefVisitor(), data );
		VisitorState currentState = (VisitorState)data;
		TypedefDeclarator declarator = currentState.getTypedefDeclarator();
		String actualType = currentState.getTypedefType();
		String safeName = SymbolTable.getSymbol( declarator.toString(),currentState.getPackage() ).getSafeName();
		MQCTypedefMapping.add(safeName, actualType);
		//Generate helper class
		return data;
	}

	/*********************************************************
	* Checks the payload of the methods and determines if 
	* payload is to high or not.
	*
	* @param Vector A vector of methods
	*
	* @return boolean true if valid, false otherwise
	*********************************************************/
	private boolean checkPayload(Vector iMethods)
	{
		Enumeration e = iMethods.elements();
		Enumeration params = null;
		InterfaceMethodParam arg = null;
		int inPayload = 0;
		int outPayload = 0;
		MQCCodeGenHelper genHelper = new MQCCodeGenHelper(config);
		InterfaceMethod iMethod = null;
				
		while(e.hasMoreElements())
		{
			iMethod = (InterfaceMethod) e.nextElement();
			params = iMethod.paramVector.elements();
			while(params.hasMoreElements())
			{
				arg = (InterfaceMethodParam) params.nextElement();
				if( !arg.paramInOut.equals("out") )
				{
					inPayload += genHelper.getParamSize(arg.paramType);
				}
				else if(!arg.paramInOut.equals("in") )
				{
					outPayload += genHelper.getParamSize(arg.paramType);
				}
			}
			//Add the return data to message payload.
			outPayload += genHelper.getParamSize(iMethod.returnType);

			if(!config.isValidMSGPayload(inPayload) ||
				 !config.isValidMSGPayload(outPayload))
			{
				 System.out.println("Method payload to high for method " + iMethod.name);
				 return false;
			}
			inPayload = 0;
			outPayload = 0;
		}
		return true;
	}

	/*********************************************************
	* Checks the payload of the methods and determines if 
	* payload is to high or not.
	*
	* @param Vector A vector of methods
	*
	* @return boolean true if valid, false otherwise
	*********************************************************/
	private boolean checkDataTypes(Vector iMethods)
	{
		Enumeration e = iMethods.elements();
		Enumeration params = null;
		InterfaceMethodParam arg = null;
		InterfaceMethod iMethod = null;
				
		while(e.hasMoreElements())
		{
			iMethod = (InterfaceMethod) e.nextElement();
			params = iMethod.paramVector.elements();
			if(!config.isValidDataType(iMethod.returnType))
			{
				System.out.println("Data type not legal: " + iMethod.returnType);
				return false;
			}
			while(params.hasMoreElements())
			{
				arg = (InterfaceMethodParam) params.nextElement();
				if(!config.isValidDataType(arg.paramType))
				{
					System.out.println("Data type not legal: " + arg.paramType);
					return false;
				}
			}
		}
		return true;
	}
	
	/*********************************************************
	* Checks the number of interfaces 
	*
	* @return boolean true if valid #, false otherwise
	*********************************************************/
	private boolean checkNoInterfaces()
	{
		return config.isValidNoInterfaces(noInterfaces);
	}

	/*********************************************************
	* Checks the number of methods
	*
	* @return boolean true if valid #, false otherwise
	*********************************************************/
	private boolean checkNoMethods()
	{
		return config.isValidNoMethods(noMethods);
	}
}	
