/*
 * Copyright (c) 2003 David E. Bakken, his research students, and Washington State University.
 * Please see the file LICENSE.pdf for more details on terms of use.
 */

package ZEN_IDL.be;

import java.io.*;
import java.io.PrintWriter;
import java.util.*;
import ZEN_IDL.ast.*;
import ZEN_IDL.be.symbolTableUtils.*;
import ZEN_IDL.be.helperUtils.*;
import util.*;

/*********************************************************
* MicroQoSCORBA visitor to generate stubs and skeleton 
* code. Some of the code is borrowed from the ZEN
* IDL compiler and changed.
*
* @author Olav Haugan
* @author David McKinnon
*********************************************************/
public class MQCVisitor extends AbstractVisitor
{
  //The configuration options
  private MQCConfig config;
  //The code generator
  private MQCCodeGenerator codegenerator;
  //The constant code generator
  private MQCConstCodeGenerator constCodegenerator;
  //The exception code generator
  private MQCExceptionGenerator exceptionGen;
  
  private static final short UNDEFINED = -1;
  
  private String packageNames;

  /*********************************************************
  * Constructor. Initializes variables.
  * 
  * @param MQCConfig The configuration object
  *********************************************************/
  public MQCVisitor(MQCConfig config)
  {
    packageNames = "";
    this.config = config;
    codegenerator = new MQCCodeGenerator(config);
    constCodegenerator = new MQCConstCodeGenerator();
    exceptionGen = new MQCExceptionGenerator(config);
  }
  
  /*********************************************************
  * Visitor function for a IDL module definition. Called
  * by the visitor pattern for each module definition.
  *
  * @param ASTmodule The module node from the parse tree
  * @param Object The data object
  *********************************************************/
  public Object visit(ASTmodule node, Object data)
  {
    VisitorState mState = (VisitorState) data;
    mState.setCurrentNodeType( IDLParserTreeConstants.JJTMODULE );

    //Reset the compressed interface names and method names mapping
    MQCCompress.clear();

    //Call the visitors for a modules children
    data = node.childrenAccept(this, data);
    mState.stkPackages.pop();
    return data;
  }
  
  /*********************************************************
  * Visitor function for a IDL identifier. Called
  * by the visitor pattern for an identifier. A directory
  * is generated if the identifier is a module.
  *
  * @param ASTidentifier The identifier node from the parse tree
  * @param Object The data object
  *********************************************************/
  public Object visit(ASTidentifier node, Object data)
  {
    VisitorState mState = (VisitorState) data;
    
    String name = node.getFirstToken().toString();   
    if(mState.getCurrentNodeType() == IDLParserTreeConstants.JJTMODULE)
    {
      //name of module/package/directory
      mState.stkPackages.push( name );
      //Create new package directory
      (new File( mState.getPackagePath() )).mkdirs();
      mState.setCurrentNodeType( UNDEFINED );
    }
    data = node.childrenAccept(this, data);
    return data;
  }

  /*********************************************************
  * Visitor function for a IDL interface decl. Called
  * by the visitor pattern for each interface.
  *
  * @param ASTinterface_dcl The interface node from the parse tree
  * @param Object The data object
  *********************************************************/
  public Object visit(ASTinterface_dcl node, Object data)
  {
    VisitorState mState = (VisitorState) data;

    String packageName = mState.getPackage();
    packageNames = packageNames + packageName + " ";
    String packageStr = mState.getPackageString();
    String packagePath = mState.getPackagePath();
    String interfaceName = node.jjtGetChild(0).jjtGetChild(0).getFirstToken().toString();
    String safeName = SymbolTable.getSymbol( interfaceName, packageName ).getSafeName();
    Vector interfaceMethods = SymbolTable.getSymbol( interfaceName, packageName ).getMethods();
    Symbol interfaceInfo = SymbolTable.getSymbol( interfaceName , mState.getPackage() );
    InterfaceInheritence inheritence = interfaceInfo.getInheritenceSpec();

    // Generate "<interfaceName>+Operations.java" file 
    MQCCodeGenInterface interfaceGen = new MQCCodeGenInterface(config);
    interfaceGen.setInterfaceName(interfaceName);
    interfaceGen.setInheritence(inheritence);
    interfaceGen.setInterfaceMethods(interfaceMethods);
    interfaceGen.setSafeName(safeName);
    interfaceGen.setVisitorState(mState);
    
    /* have to do it like this to be able to insert constants */
    interfaceGen.genStartOperations();
    data = node.childrenAccept(new MQCInterfaceVisitor(interfaceGen), data);
    interfaceGen.getPW().println("};");
    interfaceGen.getPW().println("// End of file.");
    interfaceGen.getPW().flush();
    interfaceGen.getPW().close();

    codegenerator.setSafeName( safeName );
    codegenerator.setPackageName( packageName );
    codegenerator.setPackageStr( packageStr );
    codegenerator.setPackagePath( packagePath );
    codegenerator.setInterfaceName( interfaceName );

    /* Generate stub code, e.g. "_<interfaceNameStub.java" */
    codegenerator.genStubCode( interfaceMethods ); 
    
    // Generate skeleton code, e.g. "<interfaceName>POA.java" 
    codegenerator.genSkeletonCode( interfaceMethods ); 
    return data;
  }

  /*********************************************************
  * Visitor function for a IDL constant decl. Called
  * by the visitor pattern for each constant declared.
  *
  * @param ASTconst_dcl The const node from the parse tree
  * @param Object The data object
  *********************************************************/
  public Object visit(ASTconst_dcl node, Object data)
  {
    VisitorState mState = (VisitorState) data;
    
    constCodegenerator.setPackageName( mState.getPackage() );
    constCodegenerator.setPackagePath( mState.getPackagePath() );
    constCodegenerator.setPackageStr( mState.getPackageString() );
    constCodegenerator.setName( node.jjtGetChild(1).getFirstToken().toString() );
    constCodegenerator.setValue( node.jjtGetChild(2).getFirstToken().toString() );
    constCodegenerator.setType( getVariableType( (SimpleNode) node.jjtGetChild(0) ) );
    constCodegenerator.setInInterface( false );
    
    //Generate code for constants      
    constCodegenerator.generateCode();
    return data;
  }

  /*********************************************************
  * Visitor function for a IDL exception decl. Called
  * by the visitor pattern for each exception declared.
  *
  * @param ASTexcept_dcl The exception node from the parse tree
  * @param Object The data object
  *********************************************************/
  public Object visit(ASTexcept_dcl node, Object data)
  {
    VisitorState currentState = (VisitorState)data;
    
    //Gather exception information
    currentState.initializeStructVectors();
    data = node.childrenAccept( new XExceptionVisitor(), data );

    currentState = (VisitorState)data;
    String exceptionName = node.jjtGetChild(0).getFirstToken().toString();
    Vector exTypeSpecVector = currentState.getStructTypeSpecVector();
    Vector exDeclaratorVector = currentState.getStructDeclaratorVector(); 
    String safeName = SymbolTable.getSymbol( exceptionName , currentState.getPackage() ).getSafeName();

    MQCCodeGenHelper.exceptionMap.put(exceptionName, exTypeSpecVector);
    
    exceptionGen.init();
    exceptionGen.setPackageName( currentState.getPackage() );
    exceptionGen.setPackagePath( currentState.getPackagePath() );
    exceptionGen.setPackageStr( currentState.getPackageString() );
    exceptionGen.setExceptionName(exceptionName);
    exceptionGen.setExTypeSpec(exTypeSpecVector);    
    exceptionGen.setExDecl(exDeclaratorVector);
    exceptionGen.setSafeName(safeName);
    
    exceptionGen.generateException();
    exceptionGen.generateExHolder();
    exceptionGen.generateExHelper();    
    return data; 
  }

  /*********************************************************
  * Visitor function for a IDL type defs. Called
  * by the visitor pattern for each typedef declared.
  *
  * @param ASTtype_declarator The type def node from the parse tree
  * @param Object The data object
  *********************************************************/
  public Object visit(ASTtype_declarator node, Object data )
  {
    data = node.childrenAccept( new XTypedefVisitor(), data );
    VisitorState currentState = (VisitorState)data;
    TypedefDeclarator declarator = currentState.getTypedefDeclarator();
    String actualType = currentState.getTypedefType();
    String safeName = SymbolTable.getSymbol( declarator.toString(),currentState.getPackage() ).getSafeName();
//    MQCTypedefMapping.add(safeName, actualType);
    //Generate helper class
    return data;
  }

  /*********************************************************
  * Construct the type of a variable from its node
  *
  * @param SimpleNode The variable node
  * @return String The variable type
  *********************************************************/
  private String getVariableType( SimpleNode node )
  {
    Token firstTypeToken = node.getFirstToken();
    Token lastTypeToken = node.getLastToken();

    String variableType = firstTypeToken.toString();

    while( firstTypeToken != lastTypeToken )
    {
      firstTypeToken = firstTypeToken.next;
      variableType = " " + firstTypeToken.toString();
    }
    return variableType;
  }

  /*********************************************************
  * Get the name of the all the packages for which code was generated
  *        
  * @return packageNames A list of the package names (space deliminated)
  *********************************************************/
  public String getPackageNames()
  {
    return packageNames;
  }
}
