/*
 * Copyright (c) 2003 David E. Bakken, his research students, and Washington State University.
 * Please see the file LICENSE.pdf for more details on terms of use.
 */

package mqc.protocols;

import mqc.ORB;

/**
 * MQC-IOP protocol message.
 *
 * struct MQCIOPHeader {
 *    octet           magic[3];
 *    octet           endianess;
 *    unsigned long   msg_size;
 *    octet           msg_type;
 *    octet           reserved[3];
 * };
 *
 * @author Kim Christian Swenson
 */
public class MQCIOPHeader implements Marshalable
{
   public final static byte MMAGIC = 0x4D;
   public final static byte QMAGIC = 0x51;
   public final static byte CMAGIC = 0x43;

   public final static byte SIZE_OF_HDR = 12;
   public final static byte SIZE_OF_REQ_HDR = 24; //+ op. length
   public final static byte SIZE_OF_REPLY_HDR = 8;

   // MESSAGE TYPES
   public final static byte REQUEST_MSG = (byte) 0x00;
   public final static byte REPLY_MSG   = (byte) 0x01;

   protected boolean m_bigEndian;
   protected int m_size;
   protected byte m_type;

   public MQCIOPHeader()
   {
      m_size = 0;
   }

   public int getSize()
   {
      return m_size;
   }

   public void setSize( int size )
   {
      m_size = size;
   }

   public byte getType()
   {
      return m_type;
   }

   public void setType( byte type )
   {
      m_type = type;
   }

   public boolean isBigEndian()
   {
      return m_bigEndian;
   }

   public void setBigEndian( boolean bigEndian )
   {
      m_bigEndian = bigEndian;
   }

   public void marshal( CDROutputStream out )
   {
      // octet magic[3]
      out.writeOctet( MMAGIC );
      out.writeOctet( QMAGIC );
      out.writeOctet( CMAGIC );

      // octet endianess
      out.writeOctet( m_bigEndian ? (byte)0x00 : (byte)0x01 );
      out.setBigEndian( m_bigEndian );

      // unsigned long msg_size
      out.writeUnsignedLong( m_size );

      // octet msg_type
      out.writeOctet( m_type );

      // octet reserved[3]
      out.writeOctet( (byte) 0);
      out.writeOctet( (byte) 0);
      out.writeOctet( (byte) 0);
   }

   public void deMarshal( CDRInputStream in ) throws RuntimeException 
   {
      // octet magic[3]
      byte mm = in.readOctet();
      byte qm = in.readOctet();
      byte cm = in.readOctet();

      if ( mm != MMAGIC || qm != QMAGIC || cm != CMAGIC )
      {
         throw new RuntimeException( "Not a MQC Message." );
      }

      // octet endianess
      m_bigEndian = (in.readOctet() == (byte) 0x00 );
      in.setBigEndian( m_bigEndian );

      // unsigned long msg_size
      m_size = in.readUnsignedLong();

      // octet msg_type
      m_type = in.readOctet();
      if ( m_type != REQUEST_MSG && m_type != REPLY_MSG )
      {
         throw new RuntimeException( "Message type not supported." );
      }

      // octet reserved[3]
      in.readOctet( );
      in.readOctet( );
      in.readOctet( );
   }

   public static int getMsgSize( byte[] buf, boolean bigEndian )
   {
      return CDRInputStream.readInt( buf, 4, bigEndian );
   }

   public static void setMsgSize( byte[] buf, int value, boolean bigEndian )
   {
      CDROutputStream.writeInt( buf, 4, value, bigEndian );
   }

}
