/*
 * Copyright (c) 2003 David E. Bakken, his research students, and Washington State University.
 * Please see the file LICENSE.pdf for more details on terms of use.
 */

package mqc.protocols;

/**
 * MQC-IOP reply message version 1.0 protocol message.
 * 
 * Reply header:
 * 4 byte request id, 4 byte status
 *
 * struct MWCIOPReplyHeader {
 *    unsigned long    request_id;
 *    unsigned long    reply_status;
 * };
 *
 * 
 * @author Kim Christian Swenson
 *
 */
public class MQCIOPReply implements Reply
{
   protected MQCIOPHeader m_MQCIOPHeader;

   protected int m_reqID;

   protected int m_replyStatus;

   protected Body m_body;

   public MQCIOPReply( )
   {
      this( new MQCIOPHeader() );
   }

   public MQCIOPReply( MQCIOPHeader header )
   {
      m_MQCIOPHeader = header;
      header.setType( MQCIOPHeader.REPLY_MSG );
      m_body = new Body();
   }

   public int getHdrSize()
   {
      return 12;
   }

   public int getRequestID()
   {
      return m_reqID;
   }

   public void setRequestID( int id )
   {
      m_reqID = id;
   }

   public int getCompletionStatus()
   {
      return m_body.getCompletionStatus();
   }

   public void setCompletionStatus( int status )
   {
      m_body.setCompletionStatus( status );
   }

   public int getExceptionMinorCode()
   {
      return m_body.getExceptionMinorCode();
   }

   public void setExceptionMinorCode( int minor )
   {
      m_body.setExceptionMinorCode( minor );
   }

   public int getExceptionMinorVMCID()
   {
      return m_body.getExceptionMinorVMCID();
   }

   public void setExceptionMinorVMCID( int vmcid )
   {
      m_body.setExceptionMinorVMCID( vmcid );
   }

   public String getExceptionID()
   {
      return m_body.getExceptionID();
   }

   public void setExceptionID( String id )
   {
      m_body.setExceptionID( id );
   }

   public int getReplyStatus()
   {
      return m_replyStatus;
   }

   public void setReplyStatus( int status )
   {
      m_replyStatus = status;
   }

   public Body getBody()
   {
      return m_body;
   }

   public void setBody( Body body )
   {
      m_body = body;
   }

   public boolean isBigEndian()
   {
      return m_MQCIOPHeader.isBigEndian();
   }

   public void setBigEndian( boolean bigEndian )
   {
      m_MQCIOPHeader.setBigEndian( bigEndian );
   }

   public void setMsgSize( byte[] buf, int value, boolean bigEndian )
   {
      MQCIOPHeader.setMsgSize( buf, value, bigEndian );
   }

   public void marshal( CDROutputStream out )
   {
      /* TODO implement Exception handling */

      out.reset();

      boolean origAlign = out.isAlign();
      out.setAlign( false );

      // MQCIOP header
      m_MQCIOPHeader.marshal( out );

      // unsigned long request_id;
      out.writeUnsignedLong( m_reqID );

      // unsigned long reply_status;
      out.writeLong( m_replyStatus );

      // MQCIOPReply Body

      // Copy data from body buffer into underlying
      // buffer of CDROutputStream, and adjust
      // the Stream's pointer acordingly.

      // get underlying buffer
      byte[] buf = out.getBuffer();
      
      // Get bufferPointer and align it with 4-bytes
      int bufPtr = out.bytesStreamed();
      bufPtr = (bufPtr + 3) & ~3;

      // Hack ... set message size again now that we know it .. !
      m_MQCIOPHeader.setSize( bufPtr - getHdrSize() + m_body.getPtr() );
      MQCIOPHeader.setMsgSize( buf, m_MQCIOPHeader.getSize(), m_MQCIOPHeader.isBigEndian() );

      // Resize if necessary
      if ( buf.length < bufPtr + m_body.getPtr() )
      {
         byte[] tmp = new byte[bufPtr + m_body.getPtr()];
         System.arraycopy(buf, 0, tmp, 0, out.bytesStreamed() );
         buf = tmp;
      }

      // copy data
      System.arraycopy( m_body.getBuffer(), 0, buf, bufPtr, m_body.getPtr() );

      // Insert underlying buffer in case of resize
      out.setBuffer( buf );

      // Adjust buffer pointer
      out.setPtr( bufPtr + m_body.getPtr() );

      out.setAlign( origAlign );
   }

   public void deMarshal( CDRInputStream in ) throws RuntimeException
   {
      /* TODO implement Exception handling */

      in.reset();

      boolean origAlign = in.isAlign();
      in.setAlign( false );

      // MQCIOP header
      m_MQCIOPHeader.deMarshal( in );

      // unsigned long request_id;
      m_reqID = in.readUnsignedLong( );

      // unsigned long reply_status;
      m_replyStatus = in.readLong( );

      // MQCIOPReply Body

      // Copy data from underlying buffer of 
      // CDRInputStream into body buffer, and adjust
      // the Stream's pointer acordingly.

      // Get bufferPointer and align it with 4-bytes
      int bufPtr = in.bytesStreamed();
      bufPtr = (bufPtr + 3) & ~3;

      // Make sure message body's buffer is big enough
      m_body.setMinSize( m_MQCIOPHeader.getSize() - bufPtr + getHdrSize() );

      // copy body part of message from inputstream into body
      System.arraycopy( in.getBuffer(), bufPtr, m_body.getBuffer(), 0, m_MQCIOPHeader.getSize() - bufPtr + getHdrSize() );

      // Adjust buffer pointer
      in.setPtr( m_MQCIOPHeader.getSize() + getHdrSize() );

      in.setAlign( origAlign );
   }

}
