/*
 * Copyright (c) 2003 David E. Bakken, his research students, and Washington State University.
 * Please see the file LICENSE.pdf for more details on terms of use.
 */

/**
 * Cipher.java
 *
 * This file is a combination of both the javax.crypto.Cipher.java and
 * javax.crypto.CipherSpi.java, from the Cryptix JCE implementation package.
 * Combining these files was needed to avoid the runtime configuration and
 * adaptation that is requied by JCE but not desired in MQC.
 *
 * Note: Although the Cryptix files support Algorithm parameters, they
 * were removed because the MQC implementation because MQC uses macros 
 * instead of parameters to accomplish the same thing.
 *
 * @author David McKinnon
 *----
 * Id: Cipher.java,v 1.14 2000/07/28 02:06:51 gelderen Exp
 *
 * Copyright (C) 1995-2000 The Cryptix Foundation Limited.
 * All rights reserved.
 *
 * Use, modification, copying and distribution of this software is subject
 * the terms and conditions of the Cryptix General Licence. You should have
 * received a copy of the Cryptix General Licence along with this library;
 * if not, you can download a copy from http://www.cryptix.org/ .
 */
package mqc.security;


//import java.security.AlgorithmParameters;
//import java.security.InvalidAlgorithmParameterException;
import mqc.security.InvalidKeyException;
import mqc.security.Key;
//import java.security.NoSuchAlgorithmException;
//import java.security.NoSuchProviderException;
//import java.security.Provider;
import mqc.security.SecureRandom;
//import java.security.Security;
//import java.security.spec.AlgorithmParameterSpec;
//import java.security.PublicKey;
//import mqc.security.cert.Certificate;
//import java.security.cert.X509Certificate;
import java.util.Iterator;

// XXX TODO: implement state


/**
 * version Revision: 1.14
 * author  Jeroen C. van Gelderen (gelderen@cryptix.org)
 * author  Paul Waserbrot (pw@cryptix.org)
 */
public abstract class Cipher
{

   // Static variables and constants
   //...........................................................................

   /** Constants used for the init() methods */
   public static final int
      ENCRYPT_MODE = 1,
      DECRYPT_MODE = 2,
      PRIVATE_KEY  = 3,
      PUBLIC_KEY   = 4,
      SECRET_KEY   = 5,
      UNWRAP_MODE  = 6,
      WRAP_MODE    = 7;


   // Instance variables
   //...........................................................................

   /** Our name */
   private final String transformation;

   private boolean isInitialized = false;

   // Constructor
   //...........................................................................

   /**
    * Constructs a Cipher wrapping the given cipherSpi.
    * <p>
    * This method should be private, but is not for backward compatibility.
    */
   protected Cipher(String transformation)
   {
      this.transformation = transformation;
   }

   // Cihper Implementation
   //...........................................................................

   public String toString()
   {
      //return "Cipher object: " + this.transformation;
      return this.transformation;
   }


   /**
    * Returns the name of the transformation implemented by this Cipher.
    */
   public final String getAlgorithm()
   {
      return this.transformation;
   }


   /**
    * Returns the block size if this Cipher, or 0 if the underlying
    * Cipher doesn't (yet) have a block size.
    */
   public final int getBlockSize()
   {
      return engineGetBlockSize();
   }


   /**
    * Returns the maximum number of bytes that the next update() or
    * doFinal() operation can return, given the length of the input.
    * <p>
    * This basically returns the sum of buffered data, padding, and input.
    */
   public final int getOutputSize(int inputLen)
      throws IllegalStateException
   {
      if(inputLen < 0)
      throw new IllegalArgumentException("Input size must be >= 0");

      return engineGetOutputSize(inputLen);
   }


   /**
    * Returns the IV associated with this Cipher, or null if this Cipher
    * doesn't (yet) have one.
    * <p>
    * This is a rather brain damaged convenience method, because there is
    * no corresponding setIV method (you need to use the ParameterSpec crap
    * instead).
    */
   public final byte[] getIV()
   {
      return engineGetIV();
   }


   public final void init(int opmode, Key key)
      throws InvalidKeyException
   {
      engineInit(opmode, key, new SecureRandom());
      isInitialized = true;
   }


   public final void init(int opmode, Key key, SecureRandom random)
      throws InvalidKeyException
   {
      engineInit(opmode, key, random);
      isInitialized = true;
   }


   public final byte[] update(byte[] input)
      throws IllegalStateException
   {
      if(input == null)
      throw new IllegalArgumentException();

      return engineUpdate(input, 0, input.length);
   }


   public final byte[] update(byte[] input, int inputOffset, int inputLen) 
      throws IllegalStateException
   {
      return engineUpdate(input, inputOffset, inputLen);
   }


   public final int update(byte[] input, int inputOffset, int inputLen, byte[] output)
      throws IllegalStateException, ShortBufferException
   {
         return engineUpdate(input, inputOffset, inputLen, output, 0);
   }


   public final int update(byte[] input, int inputOffset, int inputLen,
                           byte[] output, int outputOffset)
      throws IllegalStateException, ShortBufferException
   {
      return engineUpdate(input, inputOffset, inputLen,
                          output, outputOffset);
   }


   public final byte[] doFinal()
      throws IllegalStateException, IllegalBlockSizeException, BadPaddingException
   {
      return engineDoFinal(null, 0, 0);
   }


   public final int doFinal(byte[] output, int outputOffset)
      throws IllegalStateException, IllegalBlockSizeException, 
             ShortBufferException, BadPaddingException
   {
      return engineDoFinal(null, 0, 0, output, outputOffset);
   }


   public final byte[] doFinal(byte[] input)
      throws IllegalStateException, IllegalBlockSizeException, BadPaddingException
   {
      return engineDoFinal(input, 0, input.length);
   }


   public final byte[] doFinal(byte[] input, int inputOffset, int inputLen)
      throws IllegalStateException, IllegalBlockSizeException, BadPaddingException
   {
      return engineDoFinal(input, inputOffset, inputLen);
   }


   public final int doFinal(byte[] input, int inputOffset, int inputLen, byte[] output)
      throws IllegalStateException, ShortBufferException,
             IllegalBlockSizeException, BadPaddingException
   {
      return engineDoFinal(input, inputOffset, inputLen, output, 0);
   }


   public final int doFinal(byte[] input, int inputOffset, int inputLen,
                            byte[] output, int outputOffset)
      throws IllegalStateException, ShortBufferException,
      IllegalBlockSizeException, BadPaddingException
   {
      return engineDoFinal(input, inputOffset, inputLen, output, outputOffset);
   }

   //----------------------------------------------
   //--- ### NO CERTIFICATE SUPPORT FOR NOW ### ---
   //----------------------------------------------
   //    public final void
   //    init(int opmode, Certificate certificate)
   //    throws InvalidKeyException
   //    {
   //        this.init(opmode, certificate, new SecureRandom());
   //        isInitialized = true;
   //    }
   //
   //    public final void
   //    init(int opmode, Certificate certificate, SecureRandom random)
   //    throws InvalidKeyException
   //    {
   //        /*
   //         * Snipped from the JCE API, will be removed when finished.
   //         *
   //         * Initializes this cipher with the public key from the given
   //         * certificate and a source of randomness.
   //         *
   //         * The cipher is initialized for one of the following four
   //         * operations: encryption, decryption, key wrapping or key
   //         * unwrapping, depending on the value of opmode.
   //         *
   //         * If the certificate is of type X.509 and has a key usage
   //         * extension field marked as critical, and the value of the key
   //         * usage extension field implies that the public key in the
   //         * certificate and its corresponding private key are not supposed
   //         * to be used for the operation represented by the value of opmode,
   //         * an InvalidKeyException is thrown.
   //         *
   //         * If this cipher requires any algorithm parameters that cannot be
   //         * derived from the public key in the given certificate, the
   //         * underlying cipher implementation is supposed to generate the
   //         * required parameters itself (using provider-specific default or
   //         * random values) if it is being initialized for encryption or key
   //         * wrapping, and raise an InvalidKeyException if it is being
   //         * initialized for decryption or key unwrapping. The generated
   //         * parameters can be retrieved using engineGetParameters or
   //         * engineGetIV (if the parameter is an IV).
   //         *
   //         * If this cipher (including its underlying feedback or padding
   //         * scheme) requires any random bytes (e.g., for parameter
   //         * generation), it will get them from random.
   //         *
   //         * Note that when a Cipher object is initialized, it loses all
   //         * previously-acquired state. In other words, initializing a Cipher
   //         * is equivalent to creating a new instance of that Cipher and
   //         * initializing it.
   //         * Parameters:
   //         * opmode - the operation mode of this cipher (this is one of the
   //         * following: ENCRYPT_MODE, DECRYPT_MODE, WRAP_MODE or UNWRAP_MODE)
   //         * certificate - the certificate
   //         * random - the source of randomness
   //         * Throws:
   //         * java.security.InvalidKeyException - if the public key in the
   //         * given certificate is inappropriate for initializing this cipher,
   //         * or this cipher is being initialized for decryption or unwrapping
   //         * keys and requires algorithm parameters that cannot be determined
   //         * from the public key in the given certificate, or the keysize of
   //         * the public key in the given certificate has a keysize that exceeds
   //         * the maximum allowable keysize (as determined by the configured
   //         * jurisdiction policy files).
   //         */
   //        if (certificate instanceof X509Certificate) {
   //            // Get the extensions
   //            Iterator it = ((X509Certificate)certificate).getCriticalExtensionOIDs().iterator();
   //            // Check to see if key usage is marked critical
   //            // go through all oids and check for OID = 2.5.29.15
   //
   //            // Check whether the opmode corresponds to the key usage
   //            boolean [] boa = ((X509Certificate)certificate).getKeyUsage();
   //            /* The ASN.1 of KeyUsage is
   //             * KeyUsage ::= BIT STRING {
   //             *   digitalSignature        (0),
   //             *   nonRepudiation          (1),
   //             *   keyEncipherment         (2),
   //             *   dataEncipherment        (3),
   //             *   keyAgreement            (4),
   //             *   keyCertSign             (5),
   //             *   cRLSign                 (6),
   //             *   encipherOnly            (7),
   //             *   decipherOnly            (8) }
   //             */
   //        }
   //        PublicKey pk = certificate.getPublicKey();
   //        this.spi.engineInit(opmode, pk, random);
   //        isInitialized = true;
   //    }
   //----------------------------------------------
   //--- ### NO CERTIFICATE SUPPORT FOR NOW ### ---
   //----------------------------------------------

   public final byte[] wrap(Key key)
      throws IllegalStateException, IllegalBlockSizeException,
             InvalidKeyException
   {
      if (!isInitialized)
         throw new IllegalStateException("Cipher not initialized!");

      return engineWrap(key);
   }

   public final Key unwrap(byte[] wrappedKey, String wrappedKeyAlgorithm, int wrappedKeyType)
      throws IllegalStateException, InvalidKeyException,
             NoSuchAlgorithmException
   {
        if (!isInitialized)
           throw new IllegalStateException("Cipher not initialized!");

        return engineUnwrap(wrappedKey, wrappedKeyAlgorithm,
                            wrappedKeyType);
    }

    // CihperSpi Abstract Methods (must be implemented by a sub-classes)
    //...........................................................................

    protected abstract void engineSetMode(String mode)
       throws NoSuchAlgorithmException;


    protected abstract void engineSetPadding(String padding)
       throws NoSuchPaddingException;


   protected abstract int engineGetBlockSize();


   protected abstract int engineGetOutputSize(int inputLen);


   protected abstract byte[] engineGetIV();


   protected abstract void engineInit(int opmode, Key key, SecureRandom random)
      throws InvalidKeyException;


   protected abstract byte[] engineUpdate(byte[] input, int inputOffset, int inputLen);


   protected abstract int engineUpdate(byte[] input, int inputOffset, int inputLen,
                                       byte[] output, int outputOffset)
      throws ShortBufferException;


   protected abstract byte[] engineDoFinal(byte[] input, int inputOffset, int inputLen)
      throws IllegalBlockSizeException, BadPaddingException;


   protected abstract int engineDoFinal(byte[] input, int inputOffset, int inputLen,
                                        byte[] output, int outputOffset)
      throws ShortBufferException, IllegalBlockSizeException, BadPaddingException;

   protected byte[] engineWrap(Key key)
      throws IllegalBlockSizeException, InvalidKeyException
   {
      throw new RuntimeException("Not implemented by MQC!");
   }

   protected Key engineUnwrap(byte[] wrappedKey, String wrappedKeyAlgorithm, int wrappedKeyType)
      throws InvalidKeyException, NoSuchAlgorithmException
   {
      throw new RuntimeException("Not implemented by MQC!");
   }


   protected int engineGetKeySize(Key key)
      throws InvalidKeyException
   {
      throw new RuntimeException("Not implemented by MQC!");
   }
}
