/* $Id: XORp.java,v 1.1 2003/06/08 06:09:52 mckinnon Exp $
 *
 * Copyright (C) 1995-2000 The Cryptix Foundation Limited.
 * All rights reserved.
 *
 * Use, modification, copying and distribution of this software is subject
 * the terms and conditions of the Cryptix General Licence. You should have
 * received a copy of the Cryptix General Licence along with this library;
 * if not, you can download a copy from http://www.cryptix.org/ .
 *--
 * The formatting of this file was adapted from the Cryptix file formats. The
 * actually logic was written by A. David McKinnon.
 */
package mqc.security.cipher;

import mqc.security.Cipher;

import mqc.security.InvalidKeyException;
import mqc.security.Key;
import mqc.security.NoSuchAlgorithmException;
import mqc.security.SecureRandom;

import mqc.security.BadPaddingException;
import mqc.security.IllegalBlockSizeException;
import mqc.security.NoSuchPaddingException;
import mqc.security.ShortBufferException;


/**
 * XORp cipher (XORp for XOR Cipher plus, ie, variable length XOR keys)
 *
 * @version $Revision: 1.1 $
 * @author  A. David McKinnon
 */
public final class XORp extends Cipher
{
   /**
    * The block size of this XORp cipher is 1.
    * (use XORp if a longer length is desired)
    */
   private static final int BLOCK_SIZE = 1;

   /** Raw XORp key value. */
   private byte[] xorKey;

   /** Length of the XORp key */
   private int xorKeyLength;

   /** Offset into the XORp key */
   private int xorKeyOffset;


   // Constructor
   //............................................................................

   public XORp()
   {
      super("XORp/ECB/NONE");
   }


   /**
    * Always throws a CloneNotSupportedException (cloning of ciphers is not
    * supported for security reasons).
    */
   public final Object clone() throws CloneNotSupportedException
   {
      throw new CloneNotSupportedException();
   }


   // Implementation of JCE methods
   //............................................................................

   protected final void engineSetMode(String mode)
      throws NoSuchAlgorithmException
   {
      if(mode.toUpperCase().equals("ECB"))
         return;
      else
         throw new NoSuchAlgorithmException();
   }


   protected final void engineSetPadding(String padding)
      throws NoSuchPaddingException
   {
      if(padding.toUpperCase().equals("NONE") ||
         padding.toUpperCase().equals("NOPADDING") )
      {
         return;
      }
      throw new NoSuchPaddingException();
   }


   /**
    * Returns the length of an input block, in bytes.
    *
    * @return the length in bytes of an input block for this cipher.
    */
   public int engineGetBlockSize ()
   {
      return BLOCK_SIZE;
   }


   /**
    * Return output size given input length. We don't buffer so this method
    * always returns inputLen.
    */
   protected final int engineGetOutputSize(int inputLen)
   {
      return inputLen;
   }


   /**
    * We don't use IVs.
    *
    * FIXME: maybe return whatever was set?
    */
   protected final byte[] engineGetIV()
   {
      return null;
   }


   protected final void engineInit(int opmode, Key key, SecureRandom random)
      throws InvalidKeyException
   {
      if( key==null )
         throw new InvalidKeyException("Key missing");

      if( !key.getFormat().toUpperCase().equals("RAW") )
         throw new InvalidKeyException("Wrong format: RAW bytes needed");

      byte[] userKey = key.getEncoded();
      if (userKey == null)
         throw new InvalidKeyException("RAW bytes missing");

      xorKey       = new byte[userKey.length];
      xorKeyLength = xorKey.length;

      switch (opmode)
      {
      case Cipher.ENCRYPT_MODE:
      case Cipher.DECRYPT_MODE:
         // XOR uses the same key for both encryption and decryption
         System.arraycopy(userKey, 0, xorKey, 0, userKey.length);
         xorKeyOffset = 0;
         break;
      default:
         throw new InvalidKeyException("Unreachable code reached");
      }
   }


   protected final int
   engineUpdate(byte[] input, int inputOffset, int inputLen,
                byte[] output, int outputOffset)
      throws ShortBufferException
   {
      if( output.length-outputOffset < inputLen )
         throw new ShortBufferException();

      return this.internalUpdate(input, inputOffset, inputLen,
                                 output, outputOffset);
   }


   protected final byte[] engineUpdate(byte[] input, int inputOffset, int inputLen)
   {
      byte[] tmp  = new byte[this.engineGetOutputSize(inputLen)];
      this.internalUpdate(input, inputOffset, inputLen, tmp, 0);
      return tmp;
   }


   protected final int
   engineDoFinal(byte[] input, int inputOffset, int inputLen,
                 byte[] output, int outputOffset)
      throws ShortBufferException, IllegalBlockSizeException, BadPaddingException
   {
      return engineUpdate(input, inputOffset, inputLen, output, outputOffset);
   }


   protected final byte[]
   engineDoFinal(byte[] input, int inputOffset, int inputLen)
      throws IllegalBlockSizeException, BadPaddingException
   {
      byte[] tmp  = new byte[this.engineGetOutputSize(inputLen)];
      this.internalUpdate(input, inputOffset, inputLen, tmp, 0);
      return tmp;
   }


   private final int
   internalUpdate(byte[] input, int inputOffset, int inputLen,
                  byte[] output, int outputOffset)
   {
      System.arraycopy(input, inputOffset, output, outputOffset, inputLen);
      for (int i = outputOffset;  i < outputOffset + inputLen;  i++)
      {
         output[i] ^= xorKey[xorKeyOffset];
         xorKeyOffset++;
         if (xorKeyOffset == xorKeyLength) 
         {
            xorKeyOffset = 0;
         }
      }
      return inputLen;
   }
}
